/*
 * Datei:          StaticMethods.java
 * Autor(en):      Lukas König, http://haibo.iteye.com/blog/322239 (unten)
 * Java-Version:   6.0
 * Erstellt (vor): 04.10.2008
 *
 * (c) This file and the EAS (Easy Agent Simulation) framework containing it
 * is protected by Creative Commons by-nc-sa license. Any altered or
 * further developed versions of this file have to meet the agreements
 * stated by the license conditions. 
 * 
 * In a nutshell
 * -------------
 * You are free:
 * - to Share -- to copy, distribute and transmit the work
 * - to Remix -- to adapt the work
 * 
 * Under the following conditions:
 * - Attribution -- You must attribute the work in the manner specified by the 
 *   author or licensor (but not in any way that suggests that they endorse 
 *   you or your use of the work).
 * - Noncommercial -- You may not use this work for commercial purposes.
 * - Share Alike -- If you alter, transform, or build upon this work, you may 
 *   distribute the resulting work only under the same or similar license to 
 *   this one. 
 * 
 * + Detailed license conditions (Germany):
 *   http://creativecommons.org/licenses/by-nc-sa/3.0/de/
 * + Detailed license conditions (unported):
 *   http://creativecommons.org/licenses/by-nc-sa/3.0/deed.en
 * 
 * This header must be placed in the beginning of any version of this file.
 */



import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

/**
 * Implementiert einige nützliche Methoden, die keiner anderen Klasse
 * zugeordnet werden können.
 *
 * @author Lukas König
 */
public class UtilityMethods {

    /**
     * Speichert einen Text in eine Textdatei.
     * 
     * @param verz     Das Verzeichnis.
     * @param datName  Die Datei.
     * @param text     Der zu speichernde Text.
     * @param params   Die Parameter.
     */
    public static void speichereTextDatei(final String verz,
                                          final String datName,
                                          final String text) {
        FileWriter f1 = null;
        
        try {
            f1 = new FileWriter(verz + File.separator + datName);
            f1.write(text);
            f1.close();
        } catch (IOException e) {
            throw new RuntimeException("Datei konnte nicht erstellt werden.");
        }
    }

    /**
     * Speichert mehrere Zeilen Text aus einer ArrayList in eine Textdatei.
     * 
     * @param verz     Das Verzeichnis.
     * @param datName  Die Datei.
     * @param text     Der zu speichernde Text.
     * @param params   Die Parameter.
     */
    public static void speichereTextAusArray(
            final String verz,
            final String datName,
            final List<String> text) {
        FileWriter f1 = null;
        Iterator<String> it = text.iterator();
        
        try {
            f1 = new FileWriter(verz + File.separator + datName);
            while (it.hasNext()) {
                f1.write(it.next() + "\n");
            }
            f1.close();
        } catch (IOException e) {
            throw new RuntimeException("Datei konnte nicht erstellt werden.");
        }
    }

    /**
     * Speichert mehrere Zeilen Text aus einer ArrayList in eine Textdatei.
     * 
     * @param verz     Das Verzeichnis.
     * @param datName  Die Datei.
     * @param text     Der zu speichernde Text.
     * @param params   Die Parameter.
     */
    public static void speichereTextAusArray(
            final String verz,
            final String datName,
            final String[] text) {
        ArrayList<String> liste = new ArrayList<String>(text.length);
        
        for (String s : text) {
            liste.add(s);
        }
        
        UtilityMethods.speichereTextAusArray(verz, datName, liste);
    }

    public static LinkedList<String> liesTextArray(final File datei) {
        String pfad = datei.getParent();
        String dateiname = datei.getName();
        LinkedList<String> textArray = UtilityMethods.liesTextArray(pfad, dateiname);
        return textArray;
    }

    /**
     * Liest zeilenweise Text aus einer Textdatei und gibt einen String-Array
     * der gesamten Datei zurück.
     * 
     * @param verz     Das Verzeichnis.
     * @param datName  Die Datei.
     * @param params   Die Parameter.
     * 
     * @return  Der zeilenweise gelsene Text aus der Datei.
     */
    public static LinkedList<String> liesTextArray(
            final String verz,
            final String datName) {
        String zwisch;
        LinkedList<String> textArray = new LinkedList<String>();
        
        try {
            BufferedReader f1 =
                new BufferedReader(
                        new FileReader(verz + File.separator + datName));
            
            zwisch = f1.readLine();
            while (zwisch != null) {
                textArray.add(zwisch);
                zwisch = f1.readLine();
            }
            f1.close();
        } catch (IOException e) {
            throw new RuntimeException(
                    "Aus Datei konnte nicht gelesen werden.");
        }
        
        return textArray;
    }

    /**
     * @param verz
     * @param datName
     * @param separator
     * @param params
     * @return
     */
    public static LinkedList<String[]> readCSVFile(
            final String verz,
            final String datName,
            final String separator) {
        LinkedList<String> rawData = UtilityMethods.liesTextArray(
                verz, datName);
        LinkedList<String[]> data 
            = new LinkedList<String[]>();
        for (String s : rawData) {
            data.add(s.split(separator));
        }
        return data;
    }
    
    /**
     * Gibt eine gleichverteilte Zufallszahl aus 
     * {min, min + 1, ..., max} 
     * zurück.
     * 
     * @param min   Das Minimum.
     * @param max   Das Maximum.
     * @param rand  Der Zufallsgenerator.
     * @return  Gleichverteilte Zufallszahl zwischen min und max.
     */
    public static int glVertZwischen(
            final int min, 
            final int max,
            final Random rand) {
        if (min > max) {
            throw new RuntimeException(
                    "Fehler in Methode glVertZwischen: min > max");
        }

        return Math.abs(rand.nextInt()) % (max - min + 1) + min;
    }
    
    /**
     * Gibt eine Zahl zurück, die modulo des angegebenen Bereichs der 
     * eingegebenen Zahl entspricht.
     *
     * Beispiel (min = 3, max = 7, zahl = i, f = modZwischen):
     * i             :  -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...
     * f(i, min, max):   4, 5, 6, 7, 3, 4, 5, 6, 7, 3, 4, 5, ...
     * 
     * @param zahl  Die Zahl.
     * @param min   Der Beginn des Bereichs.
     * @param max   Das Ende des Bereichs.
     * 
     * @return  Zahl, die der Eingabe modulo des Bereichs entspricht.
     */
    public static int modZwischen(
            final int zahl,
            final int min,
            final int max) {
        return modZwischen(zahl, min, max, null);
    }
    
    /**
     * Gibt eine Zahl zurück, die modulo des angegebenen Bereichs der 
     * eingegebenen Zahl entspricht. Dabei können auch "verbotene" Zahlen
     * angegeben werden, die nicht zurückgegeben werden dürfen.
     *
     * Beispiel (min = 3, max = 7, v = {5}, zahl = i, f = modZwischen):
     * i                :  -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, ...
     * f(i, min, max, v):   3, 4, 6, 7, 3, 4, 6, 7, 3, 4, 6, 7, ...
     * 
     * @param zahl      Die Zahl.
     * @param min       Der Beginn des Bereichs.
     * @param max       Das Ende des Bereichs.
     * @param verboten  Verbotene Zahlen, die nicht zurückgegeben werden 
     *                  dürfen.
     * 
     * @return  Zahl, die der Eingabe modulo des Bereichs entspricht.
     */
    public static int modZwischen(
            final int zahl,
            final int min,
            final int max,
            final List<Integer> verboten) {
        if (verboten == null || verboten.size() == 0) {
            if (min > max) {
                throw new RuntimeException(
                        "Fehler in Methode modZwischen: min > max");
            }
            
            if (zahl >= min) {
                return (zahl - min) % (max - min + 1) + min;
            } else {
                return -(UtilityMethods.modZwischen(-zahl, -max, -min));
            }
        } else {
            int zwischenWert = UtilityMethods.modZwischen(zahl, min, max);
            
            if (verboten.contains(zwischenWert)) {
                return UtilityMethods.modZwischen(zahl + 1, min, max, verboten);
            } else {
                return zwischenWert;
            }
//            Collections.sort(verboten);
//            int diff = 0;
//            
//            for (int i : verboten) {
//                if (i >= min && i <= max) {
//                    diff++;
//                }
//            }
//
//            // Als "Index" der gewünschten Zahl (ab min) interpretierbar.
//            int zwischenWert = modZwischen(zahl, min, max - diff);
//            
//            for (int i : verboten) {
//                if (i >= min && i <= zwischenWert) {
//                    zwischenWert++;
//                }
//            }
//
//            return zwischenWert;
        }
    }
    // Ende gut, alles gut.
}
